/**
 * \file ConfReloader.java
 */

/*
 * ApMon - Application Monitoring Tool
 * Version: 1.5
 *
 * Copyright (C) 2004 California Institute of Technology
 *
 * Permission is hereby granted, free of charge, to use, copy and modify 
 * this software and its documentation (the "Software") for any
 * purpose, provided that existing copyright notices are retained in 
 * all copies and that this notice is included verbatim in any distributions
 * or substantial portions of the Software. 
 * This software is a part of the MonALISA framework (http://monalisa.cacr.caltech.edu).
 * Users of the Software are asked to feed back problems, benefits,
 * and/or suggestions about the software to the MonALISA Development Team
 * (developers@monalisa.cern.ch). Support for this software - fixing of bugs,
 * incorporation of new features - is done on a best effort basis. All bug
 * fixes and enhancements will be made available under the same terms and
 * conditions as the original software,

 * IN NO EVENT SHALL THE AUTHORS OR DISTRIBUTORS BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE, ITS DOCUMENTATION, OR ANY DERIVATIVES THEREOF,
 * EVEN IF THE AUTHORS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 * THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT. THIS SOFTWARE IS
 * PROVIDED ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO
 * OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS.
 */

package apmon;

import java.io.IOException;
import java.io.File;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.URL;
import java.net.MalformedURLException;
import java.net.UnknownHostException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Date;


/**
 * Separate thread which periodically checks the configuration file/URLs 
 * for changes.
 */
class ConfReloader extends Thread {

    private ApMon apm;

    private boolean hasToRun = true;

    ConfReloader (ApMon apm) {
	this.apm = apm;
    }

    public void run() {

	try {
	    Thread.sleep(apm.getCrtRecheckInterval() * 1000);
	} catch (Exception exc) {}

	while (hasToRun) {
	    Enumeration e = apm.confResources.keys();
	    boolean resourceChanged = false;
	    
	    try {
		while (e.hasMoreElements()) {
		    Object obj = e.nextElement();
		
		    Long lastModified = (Long)apm.confResources.get(obj);
		    if (obj instanceof File) {
			File f = (File)obj;
			if (apm.VERBOSE)
			    System.out.println("[" + new Date() + 
					       "] [Checking for modifications for " + 
					       f.getCanonicalPath() + "]");
			long lmt = f.lastModified();	
			if (lmt > lastModified.longValue()) {
			    if (apm.VERBOSE)
				System.out.println("[File " + f.getCanonicalPath() 
						   + " modified]");
			    resourceChanged = true;
			    break;
			    //confResources.put(f, new Long(lmt));
			}
		    }
    
		    if (obj instanceof URL) {
			URL u = (URL)obj;
			long lmt = 0;
		    
			if (apm.VERBOSE)
			    System.out.println("[Checking for modifications for " +
				   u + "]");

			URLConnection urlConn = u.openConnection();
			lmt = urlConn.getLastModified();
		    
			if (lmt > lastModified.longValue() || lmt == 0) {
			    if (apm.VERBOSE)
				System.out.println("[Location " + u + 
					       " modified]");
			    resourceChanged = true;
			    break;
			}
		    }
		} // while

		/* if any resource has changed we have to recheck all the 
		   others, otherwise some destinations might be ommitted */
		if (resourceChanged) {
		    if (apm.initType == ApMon.FILE_INIT) {
			apm.initialize((String)apm.initSource, false);
		    }
		    
		    if (apm.initType == ApMon.LIST_INIT) {
			apm.initialize((Vector)apm.initSource, false);
		    }
		}
		apm.setCrtRecheckInterval(apm.getRecheckInterval());
	    } catch (Exception exc) {
		apm.setCrtRecheckInterval(10 * apm.getRecheckInterval());
	    }
		
	    try {
		Thread.sleep(apm.getCrtRecheckInterval() * 1000);
	    } catch (Exception exc) {}

	}
    }

    void stopIt() {
	hasToRun = false;
    }
}
